@@ -1,6 +1,6 @@
 /* channel.c
  *
- * $Id: channel.c,v 1.69 2000/01/27 21:45:14 nisse Exp $
+ * $Id: channel.c,v 1.72 2000/02/06 21:43:39 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -156,15 +156,17 @@ struct lsh_string *prepare_window_adjust(struct ssh_channel *channel,
      (name exc_finish_channel_handler)
      (super exception_handler)
      (vars
-       (table object channel_table)
+       (connection object ssh_connection)
+       ;; (table object channel_table)
        ; Non-zero if the channel has already been deallocated.
        (dead . int)
        ; Local channel number 
        (channel_number . UINT32)))
 */
 
-static void do_exc_finish_channel_handler(struct exception_handler *s,
-					  const struct exception *e)
+static void
+do_exc_finish_channel_handler(struct exception_handler *s,
+			      const struct exception *e)
 {
   CAST(exc_finish_channel_handler, self, s);
 
@@ -174,10 +176,10 @@ static void do_exc_finish_channel_handler(struct exception_handler *s,
       if (self->dead)
 	werror("channel.c: EXC_FINISH_PENDING on dead channel.\n");
 
-      self->table->pending_close = 1;
+      self->connection->table->pending_close = 1;
       
-      if (!self->table->next_channel)
-	EXCEPTION_RAISE(self->super.parent, &finish_read_exception);
+      if (!self->connection->table->next_channel)
+	EXCEPTION_RAISE(self->connection->e, &finish_read_exception);
       break;
       
     case EXC_FINISH_CHANNEL:
@@ -197,7 +199,7 @@ static void do_exc_finish_channel_handler(struct exception_handler *s,
       else
 	{
 	  struct ssh_channel *channel
-	    = self->table->channels[self->channel_number];
+	    = self->connection->table->channels[self->channel_number];
 
 	  assert(channel);
 	  assert(channel->resources->super.alive);
@@ -207,13 +209,14 @@ static void do_exc_finish_channel_handler(struct exception_handler *s,
 	
 	  KILL_RESOURCE_LIST(channel->resources);
 	
-	  dealloc_channel(self->table, self->channel_number);
+	  dealloc_channel(self->connection->table, self->channel_number);
 	  self->dead = 1;
 
-	  if (self->table->pending_close && !self->table->next_channel)
+	  if (self->connection->table->pending_close &&
+	      !self->connection->table->next_channel)
 	    {
 	      /* FIXME: Send a SSH_DISCONNECT_BY_APPLICATION message? */
-	      EXCEPTION_RAISE(self->super.parent, &finish_read_exception);
+	      EXCEPTION_RAISE(self->connection->e, &finish_read_exception);
 	    }
 	}
       break;
@@ -223,7 +226,7 @@ static void do_exc_finish_channel_handler(struct exception_handler *s,
 }
 
 static struct exception_handler *
-make_exc_finish_channel_handler(struct channel_table *table,
+make_exc_finish_channel_handler(struct ssh_connection *connection,
 				UINT32 channel_number,
 				struct exception_handler *e,
 				const char *context)
@@ -233,7 +236,7 @@ make_exc_finish_channel_handler(struct channel_table *table,
   self->super.raise = do_exc_finish_channel_handler;
   self->super.context = context;
 
-  self->table = table;
+  self->connection = connection;
   self->channel_number = channel_number;
   self->dead = 0;
   
@@ -284,10 +287,10 @@ int alloc_channel(struct channel_table *table)
   
   for(i = table->next_channel; i < table->used_channels; i++)
     {
-      if (!table->in_use[i])
+      if (table->in_use[i] == CHANNEL_FREE)
 	{
 	  assert(!table->channels[i]);
-	  table->in_use[i] = 1;
+	  table->in_use[i] = CHANNEL_RESERVED;
 	  table->next_channel = i+1;
 
 	  verbose("Allocated local channel number %i\n", i);
@@ -310,6 +313,7 @@ int alloc_channel(struct channel_table *table)
       lsh_space_free(table->channels);
       table->channels = new_channels;
 
+      /* FIXME: Use realloc(). */
       new_in_use = lsh_space_alloc(new_size);
       memcpy(new_in_use, table->in_use, table->used_channels);
       lsh_space_free(table->in_use);
@@ -320,7 +324,7 @@ int alloc_channel(struct channel_table *table)
 
   table->next_channel = table->used_channels = i+1;
 
-  table->in_use[i] = 1;
+  table->in_use[i] = CHANNEL_RESERVED;
   verbose("Allocated local channel number %i\n", i);
 
   return i;
@@ -333,41 +337,73 @@ void dealloc_channel(struct channel_table *table, int i)
 
   verbose("Deallocating local channel %i\n", i);
   table->channels[i] = NULL;
-  table->in_use[i] = 0;
+  table->in_use[i] = CHANNEL_FREE;
   
   if ( (unsigned) i < table->next_channel)
     table->next_channel = i;
 }
 
+void
+use_channel(struct ssh_connection *connection,
+	    UINT32 local_channel_number)
+{
+  struct channel_table *table = connection->table;
+  struct ssh_channel *channel = table->channels[local_channel_number];
+
+  assert(channel);
+  assert(table->in_use[local_channel_number] == CHANNEL_RESERVED);
+  
+  table->in_use[local_channel_number] = CHANNEL_IN_USE;
+  verbose("Taking channel %i in use, (local %i).\n",
+	  channel->channel_number, local_channel_number);
+}
+	    
 void
 register_channel(struct ssh_connection *connection,
 		 UINT32 local_channel_number,
-		 struct ssh_channel *channel)
+		 struct ssh_channel *channel,
+		 int take_into_use)
 {
   struct channel_table *table = connection->table;
   
-  assert(table->in_use[local_channel_number]);
+  assert(table->in_use[local_channel_number] == CHANNEL_RESERVED);
   assert(!table->channels[local_channel_number]);
 
-  verbose("Taking channel %i in use, (local %i).\n",
-	  channel->channel_number, local_channel_number);
+  verbose("Registering local channel %i.\n",
+	  local_channel_number);
+  
+  /* FIXME: Is this the right place to install this exception handler? */
+  channel->e =
+    make_exc_finish_channel_handler(connection,
+				    local_channel_number,
+				    channel->e ? channel->e : connection->e,
+				    HANDLER_CONTEXT);
+
   table->channels[local_channel_number] = channel;
 
-  /* FIXME: Is this the right place to install this exception handler? */
-  channel->e = make_exc_finish_channel_handler(table,
-					       local_channel_number,
-					       connection->e,
-					       HANDLER_CONTEXT);
-
+  if (take_into_use)
+    use_channel(connection, local_channel_number);
+  
   REMEMBER_RESOURCE(connection->resources, &channel->resources->super);
 }
 
-struct ssh_channel *lookup_channel(struct channel_table *table, UINT32 i)
+struct ssh_channel *
+lookup_channel(struct channel_table *table, UINT32 i)
 {
-  return (i < table->used_channels)
+  return ( (i < table->used_channels)
+	   && (table->in_use[i] == CHANNEL_IN_USE))
     ? table->channels[i] : NULL;
 }
 
+struct ssh_channel *
+lookup_channel_reserved(struct channel_table *table, UINT32 i)
+{
+  return ( (i < table->used_channels)
+	   && (table->in_use[i] == CHANNEL_RESERVED))
+    ? table->channels[i] : NULL;
+}
+
+
 /* FIXME: It seems suboptimal to send a window adjust message for *every* write that we do.
  * A better scheme might be as follows:
  *
@@ -406,7 +442,7 @@ void channel_start_receive(struct ssh_channel *channel)
 
 /* GABA:
    (class
-     (name global_request_status)
+     (name request_status)
      (vars
        ; -1 for still active requests,
        ; 0 for failure,
@@ -414,73 +450,123 @@ void channel_start_receive(struct ssh_channel *channel)
        (status . int)))
 */
 
-static struct global_request_status *make_global_request_status(void)
+static struct request_status *
+make_request_status(void)
 {
-  NEW(global_request_status, self);
+  NEW(request_status, self);
   self->status = -1;
 
   return self;
 }
 
-/* FIXME: Split into a continuation and an exception handler */
 /* GABA:
    (class
-     (name global_request_response)
-     (super global_request_callback)
+     (name global_request_continuation)
+     (super command_continuation)
      (vars
-       (active object global_request_status)))
+       (connection object ssh_connection)
+       (active object request_status)))
 */
 
-static void
-do_global_request_response(struct global_request_callback *c,
-			   int success)
+static void 
+send_global_request_responses(struct ssh_connection *connection, 
+			      struct object_queue *q)
 {
-  CAST(global_request_response, self, c);
-  struct object_queue *q = &self->super.connection->table->active_global_requests;
-
-  assert( self->active->status == -1);
-  assert( (success == 0) || (success == 1) );
-  assert( !object_queue_is_empty(q));
-	  
-  self->active->status = success;
-
-  for (;;)
-    {
-      CAST(global_request_status, n, object_queue_peek_head(q));
-      if (!n || (n->status < 0))
-	break;
-
+   for (;;)
+     {
+       CAST(request_status, n, object_queue_peek_head(q));
+       if (!n || (n->status < 0))
+	 break;
+ 
       object_queue_remove_head(q);
 
-      /* FIXME: Perhaps install some exception handler that cancels
-       * the queue as soon as a write failes. */
-      C_WRITE(self->super.connection,
+      C_WRITE(connection,
 	      (n->status
 	       ? format_global_success()
 	       : format_global_failure()));
     }
 }
 
-static struct global_request_callback *
-make_global_request_response(struct ssh_connection *connection,
-			     struct global_request_status *active)
+static void
+do_global_request_response(struct command_continuation *s,
+			   struct lsh_object *x UNUSED)
 {
-  NEW(global_request_response, self);
+  CAST(global_request_continuation, self, s);
+  struct object_queue *q = &self->connection->table->active_global_requests;
 
-  self->super.connection = connection;
-  self->super.response = do_global_request_response;
+  assert(self->active->status == -1);
+  assert(!object_queue_is_empty(q));
+	  
+  self->active->status = 1;
 
+  send_global_request_responses(self->connection, q);
+}
+
+static struct command_continuation *
+make_global_request_response(struct ssh_connection *connection,
+			     struct request_status *active)
+{
+  NEW(global_request_continuation, self);
+
+  self->super.c = do_global_request_response;
+  self->connection = connection;
   self->active = active;
-
+   
   return &self->super;
 }
-     
-static void do_global_request(struct packet_handler *s UNUSED,
-			      struct ssh_connection *connection,
-			      struct lsh_string *packet)
-{
-  /* CAST(global_request_handler, closure, c); */
 
+
+/* GABA:
+   (class
+     (name global_request_exception_handler)
+     (super exception_handler)
+     (vars
+       (connection object ssh_connection)
+       (active object request_status)))
+*/
+
+/* NOTE: We handle *only* EXC_GLOBAL_REQUEST */
+static void 
+do_global_request_handler(struct exception_handler *c,
+			  const struct exception *e)
+{
+  CAST(global_request_exception_handler, self, c);
+  if (e->type == EXC_GLOBAL_REQUEST)
+    {
+      struct object_queue *q = &self->connection->table->active_global_requests;
+      
+      assert(self->active->status == -1);
+      assert(!object_queue_is_empty(q));
+
+      self->active->status = 0;
+  
+      send_global_request_responses(self->connection, q);
+    }
+  else
+    EXCEPTION_RAISE(c->parent, e);
+}
+
+static struct exception_handler *
+make_global_request_exception_handler(struct ssh_connection *connection,
+				      struct request_status *active,
+				      struct exception_handler *h,
+				      const char *context)
+{
+  NEW(global_request_exception_handler, self);
+
+  self->super.raise = do_global_request_handler;
+  self->super.context = context;
+  self->super.parent = h;
+  self->active = active;
+  self->connection = connection;
+  return &self->super;
+}
+
+static void
+do_global_request(struct packet_handler *s UNUSED,
+		  struct ssh_connection *connection,
+		  struct lsh_string *packet)
+{
   struct simple_buffer buffer;
   unsigned msg_number;
   int name;
@@ -494,7 +580,8 @@ static void do_global_request(struct packet_handler *s UNUSED,
       && parse_boolean(&buffer, &want_reply))
     {
       struct global_request *req;
-      struct global_request_callback *c = NULL;
+      struct command_continuation *c = &discard_continuation;
+      struct exception_handler *e = connection->e;
       
       if (!name || !(req = ALIST_GET(connection->table->global_requests,
 				     name)))
@@ -508,14 +595,25 @@ static void do_global_request(struct packet_handler *s UNUSED,
 	{
 	  if (want_reply)
 	    {
-	      struct global_request_status *a = make_global_request_status();
+	      struct request_status *a = make_request_status();
 	      
 	      object_queue_add_tail(&connection->table->active_global_requests,
 				    &a->super);
 	      
 	      c = make_global_request_response(connection, a);
+	      e = make_global_request_exception_handler(connection, a, e, HANDLER_CONTEXT);
 	    }
-	  GLOBAL_REQUEST(req, connection, &buffer, c);
+	  else
+	    {
+	      /* We should ignore failures. */
+	      static struct report_exception_info global_req_ignore =
+		STATIC_REPORT_EXCEPTION_INFO(EXC_ALL, EXC_GLOBAL_REQUEST,
+					     "Ignored:");
+	      
+	      e = make_report_exception_handler(&global_req_ignore,
+						e, HANDLER_CONTEXT);
+	    }
+	  GLOBAL_REQUEST(req, connection, name, want_reply, &buffer, c, e);
 	}
     }
   else
@@ -581,6 +679,199 @@ do_global_request_failure(struct packet_handler *s UNUSED,
 }
 
 
+/* FIXME: Don't store the channel here, instead have it passed as the
+ * argument of the continuation. This might also allow some
+ * unification with the handling of global_requests. */
+
+/* GABA:
+   (class
+     (name channel_request_continuation)
+     (super command_continuation)
+     (vars
+       (connection object ssh_connection)
+       (channel object ssh_channel)
+       (active object request_status)))
+*/
+
+static void
+send_channel_request_responses(struct ssh_connection *connection,
+			       struct ssh_channel *channel,
+			       struct object_queue *q)
+{
+  for (;;)
+    {
+      CAST(request_status, n, object_queue_peek_head(q));
+      if (!n || (n->status < 0))
+	break;
+
+      object_queue_remove_head(q);
+
+      C_WRITE(connection,
+	      (n->status
+	       ? format_channel_success(channel->channel_number)
+	       : format_channel_failure(channel->channel_number)));
+    }
+}
+
+static void
+do_channel_request_response(struct command_continuation *s,
+			    struct lsh_object *x UNUSED)
+{
+  CAST(channel_request_continuation, self, s);
+  struct object_queue *q = &self->channel->active_requests;
+
+  assert(self->active->status == -1);
+  assert(!object_queue_is_empty(q));
+	  
+  self->active->status = 1;
+
+  send_channel_request_responses(self->connection, self->channel, q);
+}
+
+static struct command_continuation *
+make_channel_request_response(struct ssh_connection *connection,
+			      struct ssh_channel *channel,
+			      struct request_status *active)
+{
+  NEW(channel_request_continuation, self);
+
+  self->super.c = do_channel_request_response;
+  self->connection = connection;
+  self->channel = channel;
+  self->active = active;
+
+  return &self->super;
+}
+
+/* GABA:
+   (class
+     (name channel_request_exception_handler)
+     (super exception_handler)
+     (vars
+       (connection object ssh_connection)
+       (channel object ssh_channel)
+       (active object request_status)))
+*/
+
+/* NOTE: We handle *only* EXC_CHANNEL_REQUEST */
+static void 
+do_channel_request_handler(struct exception_handler *c,
+			   const struct exception *e)
+{
+  CAST(channel_request_exception_handler, self, c);
+  if (e->type == EXC_CHANNEL_REQUEST)
+    {
+      struct object_queue *q = &self->channel->active_requests;
+
+      assert(self->active->status == -1);
+      assert(!object_queue_is_empty(q));
+      
+      self->active->status = 0;
+      
+      send_channel_request_responses(self->connection, self->channel, q);
+    }
+  else
+    EXCEPTION_RAISE(c->parent, e);
+}
+
+static struct exception_handler *
+make_channel_request_exception_handler(struct ssh_connection *connection,
+				       struct ssh_channel *channel,
+				       struct request_status *active,
+				       struct exception_handler *h,
+				       const char *context)
+{
+  NEW(channel_request_exception_handler, self);
+
+  self->super.raise = do_channel_request_handler;
+  self->super.parent = h;
+  self->super.context = context;
+  self->connection = connection;
+  self->channel = channel;
+  self->active = active;
+
+  return &self->super;
+}
+
+static void
+do_channel_request(struct packet_handler *closure UNUSED,
+		   struct ssh_connection *connection,
+		   struct lsh_string *packet)
+{
+  struct simple_buffer buffer;
+  unsigned msg_number;
+  UINT32 channel_number;
+  int type;
+  int want_reply;
+  
+  simple_buffer_init(&buffer, packet->length, packet->data);
+
+  if (parse_uint8(&buffer, &msg_number)
+      && (msg_number == SSH_MSG_CHANNEL_REQUEST)
+      && parse_uint32(&buffer, &channel_number)
+      && parse_atom(&buffer, &type)
+      && parse_boolean(&buffer, &want_reply))
+    {
+      struct ssh_channel *channel = lookup_channel(connection->table,
+						   channel_number);
+
+      /* NOTE: We can't free packet yet, because it is not yet fully
+       * parsed. There may be some more arguments, which are parsed by
+       * the CHANNEL_REQUEST method below. */
+
+      if (channel)
+	{
+	  struct channel_request *req;
+	  struct command_continuation *c = &discard_continuation;
+	  struct exception_handler *e = channel->e;
+
+	  if (type && channel->request_types 
+	      && ( (req = ALIST_GET(channel->request_types, type)) ))
+	    {
+	      if (want_reply)
+		{
+		  struct request_status *a = make_request_status();
+		  
+		  object_queue_add_tail(&channel->active_requests,
+					&a->super);
+		  
+		  c = make_channel_request_response(connection, channel, a);
+		  e = make_channel_request_exception_handler(connection, channel, a, e, HANDLER_CONTEXT);
+		  
+		}
+	      else
+		{
+		  /* We should ignore failures. */
+		  static struct report_exception_info channel_req_ignore =
+		    STATIC_REPORT_EXCEPTION_INFO(EXC_ALL, EXC_CHANNEL_REQUEST,
+						 "Ignored:");
+		  
+		  e = make_report_exception_handler(&channel_req_ignore,
+						    e, HANDLER_CONTEXT);
+		}
+	      
+	      CHANNEL_REQUEST(req, channel, connection, type, want_reply, &buffer, c, e);
+	    }
+	  else
+	    {
+	      if (want_reply)
+		C_WRITE(connection,
+			format_channel_failure(channel->channel_number));
+	    }
+	}
+      else
+	{
+	  werror("SSH_MSG_CHANNEL_REQUEST on nonexistant channel %i\n",
+		 channel_number);
+	}
+    }
+  else
+    PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_CHANNEL_REQUEST message.");
+  
+  lsh_string_free(packet);
+}
+
+
 /* GABA:
    (class
      (name channel_open_continuation)
@@ -613,7 +904,8 @@ do_channel_open_continue(struct command_continuation *c,
   channel->write = self->connection->write;
 
   register_channel(self->connection,
-		   self->local_channel_number, channel);
+		   self->local_channel_number, channel,
+		   1);
 
   /* FIXME: Doesn't support sending extra arguments with the
    * confirmation message. */
@@ -772,58 +1064,6 @@ static void do_channel_open(struct packet_handler *c UNUSED,
   lsh_string_free(packet);
 }     
 
-static void
-do_channel_request(struct packet_handler *closure UNUSED,
-		   struct ssh_connection *connection,
-		   struct lsh_string *packet)
-{
-  struct simple_buffer buffer;
-  unsigned msg_number;
-  UINT32 channel_number;
-  int type;
-  int want_reply;
-  
-  simple_buffer_init(&buffer, packet->length, packet->data);
-
-  if (parse_uint8(&buffer, &msg_number)
-      && (msg_number == SSH_MSG_CHANNEL_REQUEST)
-      && parse_uint32(&buffer, &channel_number)
-      && parse_atom(&buffer, &type)
-      && parse_boolean(&buffer, &want_reply))
-    {
-      struct ssh_channel *channel = lookup_channel(connection->table,
-						   channel_number);
-
-      /* NOTE: We can't free packet yet, because it is not yet fully
-       * parsed. There may be some more arguments, which are parsed by
-       * the CHANNEL_REQUEST method below. */
-
-      if (channel)
-	{
-	  struct channel_request *req;
-
-	  if (type && channel->request_types 
-	      && ( (req = ALIST_GET(channel->request_types, type)) ))
-	    CHANNEL_REQUEST(req, channel, connection, want_reply, &buffer);
-	  else
-	    {
-	      if (want_reply)
-		C_WRITE(connection,
-			format_channel_failure(channel->channel_number));
-	    }
-	}
-      else
-	{
-	  werror("SSH_MSG_CHANNEL_REQUEST on nonexistant channel %i\n",
-		 channel_number);
-	}
-    }
-  else
-    PROTOCOL_ERROR(connection->e, "Invalid SSH_MSG_CHANNEL_REQUEST message.");
-  
-  lsh_string_free(packet);
-}
-      
 static void
 do_window_adjust(struct packet_handler *closure UNUSED,
 		 struct ssh_connection *connection,
@@ -1172,20 +1412,26 @@ do_channel_open_confirm(struct packet_handler *closure UNUSED,
       && parse_uint32(&buffer, &max_packet)
       && parse_eod(&buffer))
     {
-      struct ssh_channel *channel = lookup_channel(connection->table,
-						   local_channel_number);
+      struct ssh_channel *channel =
+	lookup_channel_reserved(connection->table,
+				local_channel_number);
 
       lsh_string_free(packet);
 
-      if (channel && channel->open_continuation)
+      if (channel) 
 	{
 	  struct command_continuation *c = channel->open_continuation;
+	  assert(c);
+
 	  channel->open_continuation = NULL;
-	  
+
 	  channel->channel_number = remote_channel_number;
 	  channel->send_window_size = window_size;
 	  channel->send_max_packet = max_packet;
 
+	  /* FIXME: Initialize channel->write here? */
+	  use_channel(connection, local_channel_number);
+
 	  COMMAND_RETURN(c, channel);
 	}
       else
@@ -1228,19 +1474,22 @@ do_channel_open_failure(struct packet_handler *closure UNUSED,
       && parse_string(&buffer, &language_length, &language)
       && parse_eod(&buffer))
     {
-      struct ssh_channel *channel = lookup_channel(connection->table,
-						   channel_number);
+      struct ssh_channel *channel =
+	lookup_channel_reserved(connection->table,
+				channel_number);
 
       lsh_string_free(packet); 
 
-      if (channel && channel->open_continuation)
+      if (channel)
 	{
 	  static const struct exception finish_exception
 	    = STATIC_EXCEPTION(EXC_FINISH_CHANNEL, "CHANNEL_OPEN failed.");
 
+	  assert(channel->open_continuation);
+	  
 	  /* FIXME: It would be nice to pass the message on. */
 	  EXCEPTION_RAISE(channel->e,
-			  make_channel_open_exception(reason, "Refused by server"));
+			  make_channel_open_exception(reason, "Refused by peer"));
 	  EXCEPTION_RAISE(channel->e, &finish_exception);
 	}
       else
@@ -1487,6 +1736,7 @@ void init_channel(struct ssh_channel *channel)
   channel->resources = empty_resource_list();
   
   object_queue_init(&channel->pending_requests);
+  object_queue_init(&channel->active_requests);
 }
 
 struct lsh_string *channel_transmit_data(struct ssh_channel *channel,
@@ -1610,13 +1860,13 @@ make_channel_write_extended(struct ssh_channel *channel,
   return &closure->super.super;
 }
 
-struct io_read_callback *
+struct io_callback *
 make_channel_read_data(struct ssh_channel *channel)
 {
   return make_read_data(channel, make_channel_write(channel));
 }
 
-struct io_read_callback *
+struct io_callback *
 make_channel_read_stderr(struct ssh_channel *channel)
 {
   return make_read_data(channel,
@@ -1719,34 +1969,25 @@ make_channel_io_exception_handler(struct ssh_channel *channel,
 }
 
 struct lsh_string *
-prepare_channel_open(struct ssh_connection *connection,
-		     int type, struct ssh_channel *channel,
-		     const char *format, ...)
+format_channel_open(int type, UINT32 local_channel_number,
+		    struct ssh_channel *channel,
+		    const char *format, ...)
 {
-  int index;
-    
   va_list args;
   UINT32 l1, l2;
   struct lsh_string *packet;
   
 #define OPEN_FORMAT "%c%a%i%i%i"
-#define OPEN_ARGS SSH_MSG_CHANNEL_OPEN, type, (UINT32) index, \
+#define OPEN_ARGS SSH_MSG_CHANNEL_OPEN, type, local_channel_number, \
   channel->rec_window_size, channel->rec_max_packet  
 
-  debug("prepare_channel_open: rec_window_size = %i,\n"
-	"                      rec_max_packet = %i,\n"
-	"                      max_packet = %i\n",
+  debug("format_channel_open: rec_window_size = %i,\n"
+	"                     rec_max_packet = %i,\n"
+	"                     max_window = %i\n",
 	channel->rec_window_size,
 	channel->rec_max_packet,
 	channel->max_window);
   
-  index = alloc_channel(connection->table);
-
-  if (index < 0)
-    return 0;
-
-  register_channel(connection, index, channel);
-
   l1 = ssh_format_length(OPEN_FORMAT, OPEN_ARGS);
   
   va_start(args, format);
