@@ -2,11 +2,11 @@
  *
  * Lookup signature verifiers of a public key
  *
- * $Id: lookup_verifier.h,v 1.2 1999/11/03 22:13:57 nisse Exp $ */
+ * $Id: lookup_verifier.h,v 1.4 1999/11/16 22:13:39 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
- * Copyright (C) 1999 Balazs Scheidler
+ * Copyright (C) 1999 Balzs Scheidler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -48,10 +48,11 @@
      (name lookup_verifier)
      (vars
        (lookup method (object verifier)
+                      "int method"
                       "struct lsh_string *keyholder"
 		      "struct lsh_string *key")))
 */
 
-#define LOOKUP_VERIFIER(l, kh, key) ((l)->lookup((l), (kh), (key)))
+#define LOOKUP_VERIFIER(l, m, kh, key) ((l)->lookup((l), (m), (kh), (key)))
 
 #endif /* LSH_LOOKUP_VERIFIER_H_INCLUDED */
