@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: read_data.c,v 1.26 1999/10/02 20:03:31 nisse Exp $ */
+ * $Id: read_data.c,v 1.27 2000/02/05 17:20:06 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -25,6 +25,7 @@
 
 #include "read_data.h"
 
+#include "io.h"
 #include "werror.h"
 #include "xalloc.h"
 
@@ -63,8 +64,8 @@ static UINT32 do_read_data_query(struct io_consuming_read *s)
 }
 
 
-struct io_read_callback *make_read_data(struct ssh_channel *channel,
-					struct abstract_write *write)
+struct io_callback *make_read_data(struct ssh_channel *channel,
+				   struct abstract_write *write)
 {
   NEW(read_data, self);
 
