@@ -2,7 +2,7 @@
  *
  * A read handler for application data.
  *
- * $Id: read_data.h,v 1.11 1999/10/11 21:02:36 nisse Exp $ */
+ * $Id: read_data.h,v 1.12 2000/02/05 17:20:07 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -28,8 +28,8 @@
 
 #include "channel.h"
 
-struct io_read_callback *make_read_data(struct ssh_channel *channel,
-					struct abstract_write *write);
+struct io_callback *make_read_data(struct ssh_channel *channel,
+				   struct abstract_write *write);
 
 #if 0
 struct exception_handler *
