@@ -1,6 +1,6 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.23 2000/01/12 23:03:05 nisse Exp $
+ * $Id: server_session.c,v 1.25 2000/02/06 22:23:36 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -170,11 +170,11 @@ struct resource *make_process_resource(pid_t pid, int signal)
        (term string)
 
        ; Child process's stdio 
-       (in object io_fd)
-       (out object io_fd)
+       (in object lsh_fd)
+       (out object lsh_fd)
        ;; err may be NULL, if there's no separate stderr channel.
        ;; This happens if we use a pty, and the bash workaround
-       (err object io_fd)))
+       (err object lsh_fd)))
 */
 
 /* Receive channel data */
@@ -205,17 +205,17 @@ do_send(struct ssh_channel *s,
 {
   CAST(server_session, session, s);
 
-  assert(session->out->super.read);
+  assert(session->out->read);
   /* assert(session->out->handler); */
 
-  session->out->super.want_read = 1;
+  session->out->want_read = 1;
 
   if (session->err)
     {
-      assert(session->err->super.read);
+      assert(session->err->read);
       /* assert(session->err->handler); */
   
-      session->err->super.want_read = 1;
+      session->err->want_read = 1;
     }
 }
 
@@ -617,12 +617,18 @@ static int make_pty(struct pty_info *pty UNUSED,
 
 #define USE_LOGIN_DASH_CONVENTION 1
 
+static struct exception shell_request_failed =
+STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "Shell request failed");
+
 static void
 do_spawn_shell(struct channel_request *c,
 	       struct ssh_channel *channel,
-	       struct ssh_connection *connection,
-	       int want_reply,
-	       struct simple_buffer *args)
+	       struct ssh_connection *connection UNUSED,
+	       UINT32 type UNUSED,
+	       int want_reply UNUSED,
+	       struct simple_buffer *args,
+	       struct command_continuation *s,
+	       struct exception_handler *e)
 {
   CAST(shell_request, closure, c);
   struct server_session *session = (struct server_session *) channel;
@@ -637,7 +643,7 @@ do_spawn_shell(struct channel_request *c,
 
   if (!parse_eod(args))
     {
-      PROTOCOL_ERROR(connection->e, "Invalid shell CHANNEL_REQUEST message.");
+      PROTOCOL_ERROR(e, "Invalid shell CHANNEL_REQUEST message.");
       return;
     }
     
@@ -856,7 +862,7 @@ do_spawn_shell(struct channel_request *c,
 	    = make_channel_read_close_callback(channel);
 
 	  session->in
-	    = io_write(make_io_fd(closure->backend, in[1],
+	    = io_write(make_lsh_fd(closure->backend, in[1],
 				  io_exception_handler),
 		       SSH_MAX_PACKET, NULL);
 	  
@@ -867,12 +873,12 @@ do_spawn_shell(struct channel_request *c,
 	   * which will close the channel on read errors, or is it
 	   * better to just send EOF on read errors? */
 	  session->out
-	    = io_read(make_io_fd(closure->backend, out[0], io_exception_handler),
+	    = io_read(make_lsh_fd(closure->backend, out[0], io_exception_handler),
 		      make_channel_read_data(channel),
 		      read_close_callback);
 	  session->err 
 	    = ( (err[0] != -1)
-		? io_read(make_io_fd(closure->backend, err[0], io_exception_handler),
+		? io_read(make_lsh_fd(closure->backend, err[0], io_exception_handler),
 			  make_channel_read_stderr(channel),
 			  read_close_callback)
 		: NULL);
@@ -892,16 +898,14 @@ do_spawn_shell(struct channel_request *c,
 	/* FIXME: How to do this properly if in and out may use the
 	 * same fd? */
 	REMEMBER_RESOURCE
-	  (channel->resources, &session->in->super.super);
+	  (channel->resources, &session->in->super);
 	REMEMBER_RESOURCE
-	  (channel->resources, &session->out->super.super);
+	  (channel->resources, &session->out->super);
 	if (session->err)
 	  REMEMBER_RESOURCE
-	    (channel->resources, &session->err->super.super);
+	    (channel->resources, &session->err->super);
 
-	if (want_reply)
-	  A_WRITE(channel->write,
-		  format_channel_success(channel->channel_number) );
+	COMMAND_RETURN(s, NULL);
 
 	channel_start_receive(channel);
 	return;
@@ -914,9 +918,7 @@ do_spawn_shell(struct channel_request *c,
     close(in[1]);
   }
  fail:
-  if (want_reply)
-    A_WRITE(channel->write,
-	    format_channel_failure(channel->channel_number));
+  EXCEPTION_RAISE(e, &shell_request_failed);
 }
 
 struct channel_request *make_shell_handler(struct io_backend *backend,
@@ -932,13 +934,20 @@ struct channel_request *make_shell_handler(struct io_backend *backend,
 }
 
 #if WITH_PTY_SUPPORT
+
+static struct exception pty_request_failed =
+STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "pty request failed");
+
 /* pty_handler */
 static void
 do_alloc_pty(struct channel_request *c UNUSED,
 	     struct ssh_channel *channel,
 	     struct ssh_connection *connection UNUSED,
-	     int want_reply,
-	     struct simple_buffer *args)
+	     UINT32 type UNUSED,
+	     int want_reply UNUSED,
+	     struct simple_buffer *args,
+	     struct command_continuation *s,
+	     struct exception_handler *e)
 {
   UINT32 width, height, width_p, height_p;
   UINT8 *mode;
@@ -988,9 +997,8 @@ do_alloc_pty(struct channel_request *c UNUSED,
 		  REMEMBER_RESOURCE(channel->resources, &pty->super);
 
 		  verbose(" granted.\n");
-		  if (want_reply)
-		    A_WRITE(channel->write,
-			    format_channel_success(channel->channel_number) );
+		  COMMAND_RETURN(s, NULL);
+
 		  return;
 		}
 	      else
@@ -1004,9 +1012,7 @@ do_alloc_pty(struct channel_request *c UNUSED,
   verbose("Pty allocation failed.\n");
   lsh_string_free(term);
 
-  if (want_reply)
-    A_WRITE(channel->write,
-	    format_channel_failure(channel->channel_number) );
+  EXCEPTION_RAISE(e, &pty_request_failed);
 }
 
 struct channel_request *make_pty_handler(void)
