@@ -1,6 +1,6 @@
 /* server_userauth.h
  *
- * $Id: server_userauth.h,v 1.4 1999/11/29 21:14:23 nisse Exp $
+ * $Id: server_userauth.h,v 1.5 2000/01/12 23:07:43 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -128,4 +128,11 @@ struct command *make_userauth_service(struct int_list *advertised_methods,
 				      struct alist *methods,
 				      struct alist *services);
 
+struct exception_handler *
+make_exc_userauth_handler(struct ssh_connection *connection,
+			  struct int_list *advertised_methods,
+			  unsigned attempts,
+			  struct exception_handler *parent,
+			  const char *context);
+
 #endif /* LSH_SERVER_USERAUTH_H_INCLUDED */
