@@ -1,6 +1,6 @@
 /* tcpforward_commands.c
  *
- * $Id: tcpforward_commands.c,v 1.16 2000/01/26 22:23:49 nisse Exp $
+ * $Id: tcpforward_commands.c,v 1.20 2000/02/07 20:57:06 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -26,6 +26,7 @@
 
 #include "atoms.h"
 #include "channel_commands.h"
+#include "connection_commands.h"
 #include "format.h"
 #include "io_commands.h"
 #include "ssh.h"
@@ -58,8 +59,16 @@ static struct command make_tcpip_forward_handler;
 #define DIRECT_TCPIP_HANDLER (&make_direct_tcpip_handler.super)
 #define INSTALL_DIRECT_TCPIP (&install_direct_tcpip_handler.super.super.super)
 
+static struct catch_report_collect catch_channel_open;
+#define CATCH_CHANNEL_OPEN (&catch_channel_open.super.super.super)
+
 #include "tcpforward_commands.c.x"
 
+static struct report_exception_info open_tcpip_report =
+STATIC_REPORT_EXCEPTION_INFO(EXC_ALL, EXC_CHANNEL_OPEN, "Failed to open tcpip channel");
+
+static struct catch_report_collect catch_channel_open
+= STATIC_CATCH_REPORT(&open_tcpip_report);
 
 /* Takes a socket as argument, and returns a tcpip channel. Used by
  * the party receiving a open-tcp request, when a channel to the
@@ -74,7 +83,7 @@ do_tcpip_connect_io(struct command *ignored UNUSED,
 		    struct command_continuation *c,
 		    struct exception_handler *e UNUSED)
 {
-  CAST(io_fd, socket, x);
+  CAST(lsh_fd, socket, x);
 
   assert(socket);
 
@@ -122,7 +131,7 @@ struct command tcpip_start_io =
        (type . int)
        (max_window . UINT32)
        ; For forwarded-tcpip, port is the port listened to.
-       ; For direct-tcpip, poprt is the port to conect to.
+       ; For direct-tcpip, port is the port to connect to.
        ; In both cases, it's a port used on the server end.
        (port object address_info)
        (peer object listen_value)))
@@ -131,6 +140,7 @@ struct command tcpip_start_io =
 static struct ssh_channel *
 new_tcpip_channel(struct channel_open_command *c,
 		  struct ssh_connection *connection,
+		  UINT32 local_channel_number,
 		  struct lsh_string **request)
 {
   CAST(open_tcpip_command, self, c);
@@ -144,13 +154,13 @@ new_tcpip_channel(struct channel_open_command *c,
   channel = make_tcpip_channel(self->peer->fd, TCPIP_WINDOW_SIZE);
   channel->write = connection->write;
 
-  REMEMBER_RESOURCE(channel->resources, &self->peer->fd->super.super);
+  REMEMBER_RESOURCE(channel->resources, &self->peer->fd->super);
 
-  *request = prepare_channel_open(connection, self->type, 
-  				  channel, 
-  				  "%S%i%S%i",
-				  self->port->ip, self->port->port,
-				  self->peer->peer->ip, self->peer->peer->port);
+  *request = format_channel_open(self->type, local_channel_number,
+				 channel, 
+				 "%S%i%S%i",
+				 self->port->ip, self->port->port,
+				 self->peer->peer->ip, self->peer->peer->port);
   
   return channel;
 }
@@ -370,10 +380,13 @@ STATIC_COLLECT_1(&collect_info_remote_listen_2);
        (target object address_info))
      (expr
        (lambda (connection)
-         (listen (lambda (peer)
-	           (tcpip_start_io (open_direct_tcpip target peer connection)))
-		 backend
-	         local))))
+         (connection_remember connection
+           (listen (lambda (peer)
+	             (tcpip_start_io
+		       (catch_channel_open 
+		         (open_direct_tcpip target peer) connection)))
+		   backend
+	           local)))))
 */
 
 struct command *
@@ -392,17 +405,13 @@ make_forward_local_port(struct io_backend *backend,
 /* GABA:
    (expr
      (name forward_remote_port)
-     (globals
-       (remote_listen REMOTE_LISTEN)
-       ;; (connection_remember CONNECTION_REMEMBER)
-       (connect_io TCPIP_CONNECT_IO))
      (params
        (connect object command)
        (remote object address_info))
      (expr
        (lambda (connection)
          (remote_listen (lambda (peer)
-	                  (connect_io 
+	                  (tcpip_connect_io 
 			     ; NOTE: The use of prog1 is needed to
 			     ; delay the connect call until the
 			     ; (otherwise ignored) peer argument is
@@ -525,11 +534,13 @@ STATIC_COLLECT_1(&install_tcpip_forward_info_2.super);
          (install connection
 	   (handler (lambda (port)
 	     ;; Called when the client requests remote forwarding
+	     ;; FIXME: Doesn't catch all exceptions from listen.
              (listen (lambda (peer)
-	               ;; Called when someone connects to the
-		       ;; forwarded port.
-                       (tcpip_start_io (open_forwarded_tcpip port peer
-		                                       connection)))
+  		       ;; Called when someone connects to the
+  		       ;; forwarded port.
+  		       (tcpip_start_io
+			 (catch_channel_open 
+			   (open_forwarded_tcpip port peer) connection)))
 	             backend port)))))))
 */
 
